///
/// @file        canlib-mcp2515.h
/// @copyright   Copyright (c) 2012 Toradex AG
/// $Author: kia $
/// $Rev: 1819 $
/// $Date: 2013-09-23 13:46:32 +0200 (Mon, 23 Sep 2013) $
/// @brief       Header for MCP2515 CAN controller library 
///
/// @target      Colibri PXAxxx, T20
///

#ifndef _CANLIB_H_
#define _CANLIB_H_

#ifdef __cplusplus
extern "C" {
#endif

#define STANDARD    0x00        ///< Use standard (11 bits ID) frame types
#define EXTENDED    0x01        ///< Use extended (29 bits ID) frame types

#define PXA_TYPE     (0)        ///< PXA type
#define TEG_TYPE     (1)        ///< Tegra type
#define UNKNOWN_TYPE (2)        ///< Unknown type

/// Version info
#define CAN_VER_MAJ     1
#define CAN_VER_MIN     5
#define CAN_VER_BUILD   0

#define MSG_DATA_LENGTH     (8)

/// Structure for extended frame types (29 bit address-range)
/// Set "frameType" to "Standard" or "Extended"
/// The RTR (Remote-Frame) flag and data length code are now in separate
/// variables for better hardware independence !
typedef struct tCanMessageExt {
    int     frameType;               ///< Standard or Extended
    DWORD   id;                      ///< Message identifier (11 or 29 bits)
    BOOL    isRemoteTransmitRequest; ///< Remote Transmission Request 
    USHORT  length;                  ///< Message length (0-7)
    USHORT  place;                   ///< Message place
    ULONG   timeStamp;               ///< Should be Zero if not supported
    ULONG   errorStatus;             ///< Keeps error status 
    UCHAR   data[MSG_DATA_LENGTH];   ///< Message data
} canMessageExt;

//******************************************************************************

/// Writes the acceptance code and mask register
/// @param[in]   device      Instance number
/// @param[in]   description   New acceptance code (32bits)
/// @param[in]   mask     New acceptance mask (32bits)
/// @retval      TRUE    Success
/// @retval      FALSE   Failed cases
BOOL CANLibMCP2515WriteDescriptor(DWORD device, DWORD description, DWORD mask);

//******************************************************************************

/// Reads the acceptance code
/// @param[in]   device      Instance number
/// @retval      Acceptance code 
INT32 CANLibMCP2515ReadDescriptor(DWORD device);

//******************************************************************************

/// Reads the Mask
/// @param[in]   device      Instance number
/// @retval      Mask code 
INT32 CANLibMCP2515ReadMask(DWORD device);

//******************************************************************************

/// Resets the CAN chip
/// @param[in]   device      Instance number
/// @retval      TRUE   Success
BOOL CANLibMCP2515Reset(DWORD device);

//******************************************************************************

/// Sets the baudrate
/// @param[in]   baud    Baud rate to set
/// @retval      Success is mode change
/// @retval      FALSE   Failure
BOOL CANLibMCP2515SetBaudrate(DWORD device, DWORD baud);

//******************************************************************************

/// Sends a CAN message
/// @param[in]   device      Instance number
/// @param[in]   message     Contains message information in structure format
/// @retval      TRUE    Success
/// @retval      FALSE   Failed cases
BOOL CANLibMCP2515TransmitMessage(DWORD device, canMessageExt *message);

//******************************************************************************

/// Receives CAN messages
/// @param[in]   device      Instance number
/// @param[out]  message      Pointer to an array of CAN messages
/// @param[in]   numberOfMessages      Number of messages to receive
/// @param[in]   timeout   Wait time for receive interrupt
/// @retval      Number of received messages
DWORD CANLibMCP2515ReceiveMessages(DWORD device, canMessageExt *message, DWORD numberOfMessages, DWORD timeout);

//******************************************************************************

/// Initializes CAN Controller
/// @param[in]   device      Instance number
/// @param[in]   interruptGpio   GPIO interrupt number
/// @param[in]   waterMark   Water mark
/// @param[in]   bufferSize      Buffer size
/// @param[in]   processorType      To determine whether processor type is Tegra or PXA
/// @retval      Value depends on cases under which it is returning
INT32 CANLibMCP2515Init(DWORD device, DWORD interruptGpio, DWORD waterMark, DWORD bufferSize, DWORD processorType);

//******************************************************************************

/// De-Initializes MCP2515 controller and related functions
/// @param[in]   device      Instance number
void CANLibMCP2515DeInit(DWORD device);

//******************************************************************************

/// Returns CAN status for the particular instance
/// @param[in]   device      Instance number
/// @retval      CAN  Status information
INT32 CANLibMCP2515GetStatus(DWORD device);

//******************************************************************************

/// Prints message information
/// @param[in]   message     Contains message information in structure format
void CANLibMCP2515PrintMessage(canMessageExt *message);

//******************************************************************************

/// Fills version information
/// @param[out]  verMaj     Major version information
/// @param[out]  verMin     Minor version information
/// @param[out]  buildNumber   Build number
void CANLibMCP2515GetLibVersion(DWORD *verMaj, DWORD *verMin, DWORD *buildNumber);

//******************************************************************************

/// Resets the Interrupt flags 
/// @param[in]   device      Instance number
void CANLibMCP2515ResetInterruptFlags(DWORD device);

/// Outputs the regsister values 
/// @param[in]   device      Instance number
void DumpMcp2515(DWORD device);

#ifdef __cplusplus
}
#endif

#endif
